package cn.estsh.i3plus.ext.mes.pcn.api.busi;

import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesCellEquipContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionAssemblyNosortContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionProcessContext;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentVariable;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentVariableCfg;
import cn.estsh.i3plus.pojo.mes.bean.MesProdRouteOptParam;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import io.swagger.annotations.ApiOperation;

import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * 获取生产过程上下文对象接口【BASE】
 */
public interface IMesProductionProcessContextStepService {

    @ApiOperation(value = "清除上下文中的所有基础数据")
    void doFlushProductionProcessContext(StationRequestBean reqBean);

    @ApiOperation(value = "存储生产过程上下文对象【工步根据业务实际情况执行,配置错误等情况无需执行】")
    Boolean saveProductionProcessContext(StationRequestBean reqBean, MesProductionProcessContext productionProcessContext);

    @ApiOperation(value = "获取生产过程上下文对象")
    MesProductionProcessContext getProductionProcessContext(StationRequestBean reqBean);

    @ApiOperation(value = "获取工序工艺信息")
    MesProductionProcessContext processContext(StationRequestBean reqBean);

    @ApiOperation(value = "获取工序工艺信息")
    MesProductionProcessContext processContext(StationRequestBean reqBean, MesProductionProcessContext productionProcessContext);

    @ApiOperation(value = "获取finishCode码")
    MesProductionProcessContext finishCodeContext(StationRequestBean reqBean, MesProductionProcessContext productionProcessContext);

    @ApiOperation(value = "验证组织模型有效性")
    MesProductionProcessContext checkBaseData(StationRequestBean reqBean, MesProductionProcessContext productionProcessContext);

    @ApiOperation(value = "获取工位设备集合信息")
    MesProductionProcessContext getCellEquipmentList(StationRequestBean reqBean);

    @ApiOperation(value = "获取工位设备集合信息")
    MesProductionProcessContext getCellEquipmentList(StationRequestBean reqBean, MesProductionProcessContext productionProcessContext);

    //正常情况下均直接调用以下方法获取上下文
    @ApiOperation(value = "获取工位当前设备信息")
    MesProductionProcessContext getCurCellEquipment(StationRequestBean reqBean);

    @ApiOperation(value = "获取工位当前设备信息")
    MesProductionProcessContext getCurCellEquipment(StationRequestBean reqBean, MesProductionProcessContext productionProcessContext);

    @ApiOperation(value = "获取工位当前设备信息")
    MesProductionProcessContext getCurCellEquipment(StationRequestBean reqBean, Optional<Map<String, MesProdRouteOptParam>> stepParamMap);

    @ApiOperation(value = "获取工位当前设备信息")
    MesProductionProcessContext getCurCellEquipment(StationRequestBean reqBean, MesProductionProcessContext productionProcessContext, Optional<Map<String, MesProdRouteOptParam>> stepParamMap);

    @ApiOperation(value = "获取设备数据变量接口逻辑信息")
    MesProductionProcessContext getEquipmentVariableCfgList(StationRequestBean reqBean);

    @ApiOperation(value = "获取设备数据变量接口逻辑信息")
    MesProductionProcessContext getEquipmentVariableCfgList(MesProductionProcessContext productionProcessContext);

    @ApiOperation(value = "根据变量类别搜集设备数据变量接口逻辑信息")
    List<MesEquipmentVariableCfg> collectEquipmentVariableCfgList(List<MesEquipmentVariableCfg> equipmentVariableCfgList, String... variableCategory);

    @ApiOperation(value = "验证设备数据变量接口逻辑信息中是否存在必要的变量类型信息")
    MesProductionProcessContext checkNecessaryEquipmentVariableCfg(MesProductionProcessContext productionProcessContext, MesCellEquipContext cellEquipContext, List<MesEquipmentVariableCfg> equipmentVariableCfgList, String... variableCategory);

    @ApiOperation(value = "验证设备数据变量接口逻辑信息中是否存在必要的变量类型信息, 并且参考值不能为空")
    MesProductionProcessContext checkNecessaryEquipmentVariableCfgAndValue(MesProductionProcessContext productionProcessContext, MesCellEquipContext cellEquipContext, List<MesEquipmentVariableCfg> equipmentVariableCfgList, String... variableCategory);

    @ApiOperation(value = "搜集设备数据变量接口逻辑信息中的二级变量")
    List<String> collectCategoryLevelTwoList(List<MesEquipmentVariableCfg> equipmentVariableCfgList);

    @ApiOperation(value = "获取设备数据变量信息")
    MesProductionProcessContext getEquipmentVariableList(StationRequestBean reqBean, Integer variableType);

    @ApiOperation(value = "获取设备数据变量信息")
    MesProductionProcessContext getEquipmentVariableList(MesProductionProcessContext productionProcessContext, Integer variableType);

    @ApiOperation(value = "获取设备数据变量信息")
    List<MesEquipmentVariable> findEquipmentVariableList(MesProductionProcessContext productionProcessContext, Integer variableType);

    @ApiOperation(value = "根据变量类型与二级变量获取设备数据变量信息")
    List<MesEquipmentVariable> findEquipmentVariableList(MesProductionProcessContext productionProcessContext, Integer variableType, List<String> categoryLevelTwoList);

    @ApiOperation(value = "获取设备数据变量信息")
    List<MesEquipmentVariable> findEquipmentVariableList(List<MesEquipmentVariable> equipmentVariableList, Integer variableType, List<String> categoryLevelTwoList);

    @ApiOperation(value = "验证设备数据变量信息是否为空")
    MesProductionProcessContext checkIsEmptyEquipmentVariableList(MesProductionProcessContext productionProcessContext, MesCellEquipContext cellEquipContext, List<MesEquipmentVariable> equipmentVariableList, MesExtEnumUtil.EQUIP_VARIABLE_TYPE production, List<String> categoryLevelTwoList);

    @ApiOperation(value = "查询工位设备装配件规则信息")
    List<MesProductionAssemblyNosortContext> findAssemblyNosortCfgList(StationRequestBean reqBean);

    @ApiOperation(value = "存储设备的装配件清单")
    Boolean saveAssemblyNosortCfgContext(StationRequestBean reqBean, List<MesProductionAssemblyNosortContext> assemblyNosortCfgList);


}
