package cn.estsh.i3plus.ext.mes.pcn.apiservice.controller.busi;


import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesShippingLoadingCheckService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesRedisLockUtil;
import cn.estsh.i3plus.ext.mes.pcn.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesShippingLoadingCheckModel;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesLoadingList;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@Api(tags = "装车校验")
@RequestMapping(MesCommonConstant.MES_YANFEN + "/shipping-loading-check")
@Slf4j
public class MesShippingLoadingCheckController {

    @Autowired
    private IMesShippingLoadingCheckService shippingLoadingCheckService;

    @Autowired
    private MesRedisLockUtil redisLockUtil;

    @GetMapping("/query-loading-order")
    @ApiOperation(value = "查询装车单")
    public ResultBean queryLoadingOrderNo(MesLoadingList loadingVehiclesOrder) {

        String moduleKey = "shipping:loadingOrderNo:" + loadingVehiclesOrder.getOrderCode();

        //加锁
        boolean lockFlg = false;
        try {

            //加锁
            lockFlg = redisLockUtil.redisLock(moduleKey);

            // 数据校验
            ValidatorBean.checkNotNull(loadingVehiclesOrder.getOrderCode(), "装车单不能为空");
            loadingVehiclesOrder.setOrganizeCode(AuthUtil.getOrganizeCode());

            MesShippingLoadingCheckModel model = shippingLoadingCheckService.queryLoadingOrderNo(loadingVehiclesOrder);
            return ResultBean.success("查询成功").setResultObject(model);
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        } finally {
            //解锁
            redisLockUtil.redisUnLock(lockFlg, moduleKey);
        }
    }

    @PostMapping("/scan-loading-order-detail")
    @ApiOperation(value = "扫描装车单明细-发运单号")
    public ResultBean saveLoadingOrderNoDetail(@RequestBody MesShippingLoadingCheckModel model) {

        String moduleKey = "shipping:shippingOrderNo:" + model.getShippingCode();

        //加锁
        boolean lockFlg = false;
        try {

            //加锁
            lockFlg = redisLockUtil.redisLock(moduleKey);

            // 数据校验
            ValidatorBean.checkNotNull(model.getShippingCode(), "发运单号不能为空");
            ValidatorBean.checkNotNull(model.getDetailList(), "装车单明细不能为空");
            String org = AuthUtil.getOrganizeCode();
            shippingLoadingCheckService.saveVehiclesOrderAndDetail(model, org);
            return ResultBean.success("查询成功").setResultObject(model);
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        } finally {
            //解锁
            redisLockUtil.redisUnLock(lockFlg, moduleKey);
        }
    }

    @GetMapping("/query-auto-push-order")
    @ApiOperation(value = "自动推单")
    public ResultBean queryAutoPushOrder(String shippingGroupNo) {

        try {

//            MesLoadingList order = shippingLoadingCheckService.queryLoadingOrderNoByAutoPushOrder(shippingGroupNo, "CK01");
            MesLoadingList order = shippingLoadingCheckService.queryLoadingOrderNoByAutoPushOrder(shippingGroupNo, AuthUtil.getOrganizeCode());
            return ResultBean.success("查询成功").setResultObject(order);
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
