package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesEquipmentLogService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.equiplog.MesEquipmentLogExtService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.ActorMessage;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.codemaker.SnowflakeIdMaker;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentLog;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentLogDetail;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentRecord;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentVariable;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentLogDetailRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentRecordRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
public class MesEquipmentLogService implements IMesEquipmentLogService {

    public static final Logger LOGGER = LoggerFactory.getLogger(CommonConstWords.SYSTEM_LOG);

    @Autowired
    private MesEquipmentLogExtService mesEquipmentLogExtService;
    @Autowired
    private MesEquipmentExtService mesEquipmentExtService;
    @Autowired
    private MesEquipmentLogDetailRepository mesEquipmentLogDetailRepository;
    @Autowired
    private MesEquipmentRecordRepository mesEquipmentRecordRepository;
    @Autowired
    private SnowflakeIdMaker snowflakeIdMaker;
    @Resource(name = "redisMesPcn")
    private ImppRedis redisMesPcn;

    @Override
    public void saveEquipmentLogToRedis(ActorMessage actorMessage) {
        // 获取设备变量属性
        List<MesEquipmentVariable> equipmentVariableList = mesEquipmentExtService.getEquipmentVariableList(actorMessage.getOrganizeCode(), actorMessage.getEquipmentCode(), MesExtEnumUtil.EQUIP_VARIABLE_TYPE.PROCESS_FINISH.getValue());
        // 获取变量ID集合
        List<Long> equipVariableIdList = equipmentVariableList.stream().filter(o -> null != o).map(MesEquipmentVariable::getId).collect(Collectors.toList());
        // 获取设备数据采集信息
        List<MesEquipmentLog> equipmentLogList = mesEquipmentLogExtService.getEquipmentLogList(actorMessage.getOrganizeCode(), actorMessage.getEquipId());
        // 筛选出该设备中设备变量属性对应的数据采集信息
        equipmentLogList = equipmentLogList.stream().filter(mesEquipmentLog -> equipVariableIdList.contains(mesEquipmentLog.getEquipVariableId())).collect(Collectors.toList());

        LOGGER.info("需要保存的工艺参数信息: list ={}", equipmentLogList);

        Long recordId = snowflakeIdMaker.nextId();

        MesEquipmentRecord mesEquipmentRecord =new MesEquipmentRecord();
        BeanUtils.copyProperties(actorMessage, mesEquipmentRecord);
        mesEquipmentRecord.setRecordStatus(MesExtEnumUtil.RECORD_STATUS.RECORD_STATUS_10.getValue());
        mesEquipmentRecord.setId(recordId);

        ConvertBean.serviceModelInitialize(mesEquipmentRecord, actorMessage.getUserName());

        String key = getKey(mesEquipmentRecord);
        LOGGER.info("key ={}", key);
        // todo 目前没有适合的方法，需要优化一下
        equipmentLogList.forEach(equipLog -> {

            redisMesPcn.putList(key, equipLog);
        });
     /*   MesEq
        redisMesPcn.putObject()*/
        mesEquipmentRecordRepository.save(mesEquipmentRecord);
        LOGGER.info("产线【{}】,工位【{}】,设备【{}】下，保存rediskey【{}】和记录成功", mesEquipmentRecord.getWorkCenterCode(),mesEquipmentRecord.getWorkCellCode(),mesEquipmentRecord.getEquipmentCode(),key);
    }

    private String getKey(MesEquipmentRecord mesEquipmentRecord){
        return mesEquipmentRecord.getOrganizeCode() + ":" + mesEquipmentRecord.getWorkCenterCode() + ":" + mesEquipmentRecord.getWorkCellCode() + ":" + mesEquipmentRecord.getEquipmentCode () + ":" + mesEquipmentRecord.getId();
    }

    @Override
    public void saveEquipmentLogDetails(String organizeCode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.RECORD_STATUS.RECORD_STATUS_10.getValue(), "recordStatus", ddlPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue()}, new String[]{"createDatetime"}, ddlPackBean);
        List<MesEquipmentRecord> mesEquipmentRecords = mesEquipmentRecordRepository.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(mesEquipmentRecords)) {
            LOGGER.info("没有设备记录需要同步");
        }
        LOGGER.info("读取设备记录信息【{}】条", mesEquipmentRecords.size());
        for (MesEquipmentRecord mesEquipmentRecord : mesEquipmentRecords) {
            saveEquipmentLogDetail(mesEquipmentRecord);
        }
    }

    private void saveEquipmentLogDetail(MesEquipmentRecord mesEquipmentRecord) {

        String key = getKey(mesEquipmentRecord);
        List<MesEquipmentLog> equipmentLogList = (List<MesEquipmentLog>)redisMesPcn.getList(key, 0, -1);
        if (CollectionUtils.isEmpty(equipmentLogList)) {
            // 修改状态
            mesEquipmentRecord.setRecordStatus(MesExtEnumUtil.RECORD_STATUS.RECORD_STATUS_20.getValue());
            mesEquipmentRecordRepository.save(mesEquipmentRecord);
            // 删除key
            redisMesPcn.deleteKey(key);
            return;
        }
        List<MesEquipmentLogDetail> mesEquipmentLogDetails = new ArrayList<>();

        for (MesEquipmentLog mesEquipmentLog : equipmentLogList) {
            MesEquipmentLogDetail mesEquipmentLogDetail = new MesEquipmentLogDetail();
            BeanUtils.copyProperties(mesEquipmentLog, mesEquipmentLogDetail);
            mesEquipmentLogDetail.setAreaCode(mesEquipmentRecord.getAreaCode());
            mesEquipmentLogDetail.setCraftCode(mesEquipmentRecord.getCraftCode());
            mesEquipmentLogDetail.setProcessCode(mesEquipmentRecord.getProcessCode());
            mesEquipmentLogDetail.setWorkCenterCode(mesEquipmentRecord.getWorkCenterCode());
            mesEquipmentLogDetail.setId(null);
            mesEquipmentLogDetail.setCreateDatetime(null);
            mesEquipmentLogDetail.setCreateUser(null);


            String jsonStr = mesEquipmentRecord.getProduceSnJson();
            if (!StringUtils.isEmpty(jsonStr)) {

                mesEquipmentLogDetail.setExtend(mesEquipmentRecord.getProduceSnJson());
            }
            mesEquipmentLogDetails.add(mesEquipmentLogDetail);
        }
        mesEquipmentLogDetailRepository.saveAll(mesEquipmentLogDetails);
        // 修改状态
        mesEquipmentRecord.setRecordStatus(MesExtEnumUtil.RECORD_STATUS.RECORD_STATUS_20.getValue());
        mesEquipmentRecordRepository.save(mesEquipmentRecord);
        // 删除key
        redisMesPcn.deleteKey(key);
    }
}
