package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesPartService;
import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesTemplateService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProduceSnPrintService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesProduceSnPrintDataModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesProduceSnPrintModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.api.iservice.base.IConfigService;
import cn.estsh.i3plus.mes.pcn.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.mes.pcn.util.DateUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.codemaker.SnowflakeIdMaker;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Description : 条码补打
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/6/13 15:52
 * @Modify:
 **/
@Service
public class MesProduceSnPrintService implements IMesProduceSnPrintService {

    @Autowired
    private MesPackingDefineDetailsRepository mesPackingDefineDetailsRDao;

    @Autowired
    private MesPackingDefineRepository mesPackingDefineRDao;

    @Autowired
    private MesProduceSnRepository mesProduceSnRDao;

    @Autowired
    private SnowflakeIdMaker snowflakeIdMaker;

    @Autowired
    private IMesPartService mesPartService;

    @Autowired
    private IMesTemplateService mesTemplateService;

    @Autowired
    private MesCustomerPartRepository mesCustomerPartRDao;


    @Autowired
    private IConfigService configService;

    @Autowired
    private ISyncFuncService syncFuncService;

    @Autowired
    private MesNumberRuleRepository mesNumberRuleRDao;

    @Autowired
    private IMesPrintedSnLogRepository mesPrintedSnRao;

    @Override
    public MesProduceSnPrintModel findMesPackingDefine(MesProduceSnPrintModel mesProduceSnPrintModel) {
        //查询SAP下发物料信息
        MesPartSap mesPartSap = mesPartService.getMesPartSapByPartNo(mesProduceSnPrintModel.getPartNo(), mesProduceSnPrintModel.getOrganizeCode());
        mesProduceSnPrintModel.setQty(mesPartSap.getSnp());
        mesProduceSnPrintModel.setPartName(mesPartSap.getPartName());
        //查询包装定义明细
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesProduceSnPrintModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesProduceSnPrintModel.getPartNo(), "partNo", ddlPackBean);
        List<MesPackingDefineDetails> packingDefineDetails = mesPackingDefineDetailsRDao.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(packingDefineDetails)) {
            MesPcnException.throwMesBusiException("物料【%s】未维护包装定义信息", mesProduceSnPrintModel.getPartNo());
        }
        //查询包装定义主表
        ddlPackBean = DdlPackBean.getDdlPackBean(mesProduceSnPrintModel.getOrganizeCode());
        DdlPreparedPack.getInPackList(packingDefineDetails.stream().map(MesPackingDefineDetails::getPackCode).collect(Collectors.toList()), "packCode", ddlPackBean);
        mesProduceSnPrintModel.setMesPackingDefineList(mesPackingDefineRDao.findByHqlWhere(ddlPackBean));
        return mesProduceSnPrintModel;
    }

    @Override
    public MesProduceSn findMesProduceSn(MesProduceSnPrintModel mesProduceSnPrintModel) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesProduceSnPrintModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesProduceSnPrintModel.getProductSn(), "productSn", ddlPackBean);
        MesProduceSn mesProduceSn = mesProduceSnRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesProduceSn)) {
            MesPcnException.throwMesBusiException("条码【%s】信息不存在", mesProduceSnPrintModel.getProductSn());
        }
        return mesProduceSn;
    }

    @Override
    public MesProduceSnPrintModel doPrint(MesProduceSnPrintModel mesProduceSnPrintModel) {
        // 获取工厂名称
        MesConfig mesConfig = configService.getMesConfigByCfgCode(MesPcnExtConstWords.ORGANIZE_NAME,mesProduceSnPrintModel.getOrganizeCode());
        if (null == mesConfig || StringUtils.isEmpty(mesConfig.getCfgValue())){
            throw ImppExceptionBuilder.newInstance().setSystemID(CommonEnumUtil.SOFT_TYPE.MES_PCN.getCode()).setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode()).setErrorDetail("没有维护当前工厂的系统参数!").build();
        }
        //获取工厂名称
        String organizeName = mesConfig.getCfgValue();
        //物料信息
        MesPart mesPart = mesPartService.getMesPartByPartNo(mesProduceSnPrintModel.getPartNo(), mesProduceSnPrintModel.getOrganizeCode());
        //打印日期格式
        String format = "yyyy/MM/dd";
        if (MesPcnEnumUtil.PRINT_TEMPLATE.NO_SORT_BAR_CODE_GZ.getCode().equals(mesProduceSnPrintModel.getPrintTemplate()) || MesPcnEnumUtil.PRINT_TEMPLATE.NO_SORT_BAR_CODE_GZ_NEW.getCode().equals(mesProduceSnPrintModel.getPrintTemplate())) {
            format = DateUtil.BASE_FORMAT;
        }
        //条码生成
        if (StringUtils.isEmpty(mesProduceSnPrintModel.getProductSn())) {
            ValidatorBean.checkNotNull(mesProduceSnPrintModel.getSnRule(), "条码规则不能为空");
            ValidatorBean.checkNotNull(mesProduceSnPrintModel.getPrintTemplate(), "打印模板不能为空");
            //获取条码规则,根据条码规则中的序列号长度截取序列号
            DdlPackBean mesRulePackBean = DdlPackBean.getDdlPackBean(mesProduceSnPrintModel.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(mesProduceSnPrintModel.getSnRule(),"ruleCode", mesRulePackBean);
            List<MesNumberRule> ruleList = mesNumberRuleRDao.findByHqlWhere(mesRulePackBean);
            if (ruleList.isEmpty()){
                MesPcnException.throwMesBusiException("打印模板规则查询不到!");
            }
            MesNumberRule mesNumberRule = ruleList.get(0);
            int serialNoLength = mesNumberRule.getSerialnoLength();
            GenSerialNoModel genSerialNoModel = new GenSerialNoModel();
            genSerialNoModel.setRuleCode(mesPart.getProductMatchRule());
            genSerialNoModel.setPartNo(mesPart.getPartNo());
            int printQty = mesProduceSnPrintModel.getPrintQty();
            String sn;
            for (int i = 0; i < printQty; i++) {
                ResultBean resultBean = syncFuncService.syncSerialNo(genSerialNoModel, mesProduceSnPrintModel.getUserName(), mesProduceSnPrintModel.getOrganizeCode(), 1);
                 sn = resultBean.getResultList().get(0).toString();
                //保存条码信息
                MesProduceSn produceSn = generateMesProduceSn(mesPart, sn, mesProduceSnPrintModel.getUserName(), mesProduceSnPrintModel.getQty(), format);
                //封装打印信息
                mesProduceSnPrintModel.getMesProduceSnPrintDataModelList().add(getModel(produceSn, sn.substring(sn.length()-serialNoLength), mesProduceSnPrintModel.getCustPartNo(), organizeName));
                mesProduceSnRDao.insert(produceSn);
                //保存打印记录
                MesPrintedSnLog snLog = new MesPrintedSnLog();
                ConvertBean.serviceModelInitialize(snLog, mesProduceSnPrintModel.getUserName());
                snLog.setSn(sn);
                mesPrintedSnRao.insert(snLog);
            }

            //条码补打
        } else {
            ValidatorBean.checkNotNull(mesPart.getProductLabelTemplate(), "打印模板不能为空");
            mesProduceSnPrintModel.setPrintTemplate(mesPart.getProductLabelTemplate());
            MesProduceSn mesProduceSn = findMesProduceSn(mesProduceSnPrintModel);
            String no = "";
            String custPartNo = "";
            if (MesPcnEnumUtil.PRINT_TEMPLATE.GQXNY.getCode().equals(mesProduceSnPrintModel.getPrintTemplate())
                    && !Objects.isNull(mesProduceSn.getProductSn())
                    && mesProduceSn.getProductSn().length() >= 40) {
                //获取客户零件号
                custPartNo = getMesCustomerPart(mesProduceSnPrintModel).getCustPartNo();
                no = mesProduceSn.getProductSn().substring(36, 40);
            }
            mesProduceSnPrintModel.getMesProduceSnPrintDataModelList().add(getModel(mesProduceSn, no, custPartNo, organizeName));
            MesPrintedSnLog snLog = new MesPrintedSnLog();
            ConvertBean.serviceModelInitialize(snLog, mesProduceSnPrintModel.getUserName());
            snLog.setSn(mesProduceSn.getProductSn());
            mesPrintedSnRao.insert(snLog);
        }
        //获取打印模板信息
        mesProduceSnPrintModel.setMesLabelTemplate(mesTemplateService.getMesLabelTemplate(mesProduceSnPrintModel.getPrintTemplate()));
        return mesProduceSnPrintModel;
    }

    private String getTemplate(MesProduceSnPrintModel mesProduceSnPrintModel, MesPart mesPart) {
        Date date = new Date();
        StringBuilder template = new StringBuilder();
        if (MesPcnEnumUtil.PRINT_TEMPLATE.B2M.getCode().equals(mesProduceSnPrintModel.getPrintTemplate()) || MesPcnEnumUtil.PRINT_TEMPLATE.GQAA.getCode().equals(mesProduceSnPrintModel.getPrintTemplate())) {
            //获取客户零件号
            MesCustomerPart mesCustomerPart = getMesCustomerPart(mesProduceSnPrintModel);
            mesProduceSnPrintModel.setCustPartNo(mesCustomerPart.getCustPartNo());
            //"+"（1）+零件简号（4）+ 供应商代码（SE103）+生产线代号(1)+生产日期(4)+供应商生产流水号(4) +软件版本代号(2)
            template.append("+").append(mesPart.getPartSnParam()).append("SE103").append("1").append(getDateTime(date)).append("serialNo").append("01");
        } else if (MesPcnEnumUtil.PRINT_TEMPLATE.GQXNY.getCode().equals(mesProduceSnPrintModel.getPrintTemplate())) {
            //获取客户零件号
            MesCustomerPart mesCustomerPart = getMesCustomerPart(mesProduceSnPrintModel);
            mesProduceSnPrintModel.setCustPartNo(mesCustomerPart.getCustPartNo());
            //零件号（17）+扩展位（2）+供应商代码（7）+扩展位（3）+产线（1）+年月日（6位）+流水号（4）+扩展位（4）+"+"
            template.append(stringFormat(mesCustomerPart.getCustPartNo(), 17)).append("00").append(stringFormat("VD501", 7)).append("000").append("1").append(TimeTool.pareDateToString("yyMMdd", date)).append("serialNo").append("0000").append("+");
        } else if (MesPcnEnumUtil.PRINT_TEMPLATE.NO_SORT_BAR_CODE_GZ.getCode().equals(mesProduceSnPrintModel.getPrintTemplate())) {
            //内部条码：M00061102-R619+"10158"
            //条码拼接规则：零件号+年月日+5位流水号
            template.append(mesPart.getPartNo()).append("-").append(getDateTime(date)).append("serialNo");
        } else if (MesPcnEnumUtil.PRINT_TEMPLATE.NO_SORT_BAR_CODE_GZ_NEW.getCode().equals(mesProduceSnPrintModel.getPrintTemplate())) {
            //获取客户零件号
            MesCustomerPart mesCustomerPart = getMesCustomerPart(mesProduceSnPrintModel);
            mesProduceSnPrintModel.setCustPartNo(mesCustomerPart.getCustPartNo());
            //小鹏客户条码：6205100EF3-S5-06-R61910027
            //条码拼接规则：客户零件号+中杠+年月日+5位流水号（按天递增）
            template.append(mesCustomerPart.getCustPartNo()).append("-").append(getDateTime(date)).append("serialNo");
        }
        return template.toString();
    }

    private MesProduceSnPrintDataModel getModel(MesProduceSn produceSn, String no, String custPartNo, String factoryName) {
        MesProduceSnPrintDataModel mesProduceSnPrintDataModel = new MesProduceSnPrintDataModel();
        mesProduceSnPrintDataModel.setFactoryName(factoryName);
        mesProduceSnPrintDataModel.setPartNo(produceSn.getPartNo());
        mesProduceSnPrintDataModel.setPartName(produceSn.getPartName());
        mesProduceSnPrintDataModel.setNo(no);
        mesProduceSnPrintDataModel.setBarcode(produceSn.getProductSn());
        mesProduceSnPrintDataModel.setCustPartNo(custPartNo);
        mesProduceSnPrintDataModel.setProductDate(produceSn.getLotNo());
        return mesProduceSnPrintDataModel;
    }

//    private List<String> getSerialNoList(MesProduceSnPrintModel mesProduceSnPrintModel) {
//        List<String> serialNoList = new ArrayList<>();
//        //获取系统参数
//        String configKey = MesPcnEnumUtil.PRINT_TEMPLATE.codeOfValue(mesProduceSnPrintModel.getPrintTemplate());
//        if (StringUtils.isEmpty(configKey)) {
//            MesPcnException.throwMesBusiException("打印模板未匹配到流水码规则");
//        }
//        MesConfig mesConfig = mesConfigService.getMesConfig(mesProduceSnPrintModel.getOrganizeCode(), configKey);
//        int cfgValue = StringUtils.isEmpty(mesConfig.getCfgValue()) || !CheckTool.isInteger(mesConfig.getCfgValue()) ? 1 : Integer.parseInt(mesConfig.getCfgValue());
//        if (MesPcnEnumUtil.PRINT_TEMPLATE.B2M.getCode().equals(mesProduceSnPrintModel.getPrintTemplate()) || MesPcnEnumUtil.PRINT_TEMPLATE.GQAA.getCode().equals(mesProduceSnPrintModel.getPrintTemplate())) {
//            if (StringUtils.isEmpty(mesConfig.getCfgValueDesc()) || !TimeTool.getYear().equals(mesConfig.getCfgValueDesc())) {
//                cfgValue = 1;
//                mesConfig.setCfgValueDesc(TimeTool.getYear());
//            }
//            for (int i = 0; i < mesProduceSnPrintModel.getPrintQty(); i++) {
//                serialNoList.add(stringFormat(toBase34(cfgValue), 4, false, "0"));
//                cfgValue++;
//            }
//        } else if (MesPcnEnumUtil.PRINT_TEMPLATE.GQXNY.getCode().equals(mesProduceSnPrintModel.getPrintTemplate())) {
//            for (int i = 0; i < mesProduceSnPrintModel.getPrintQty(); i++) {
//                serialNoList.add(String.format("%04d", cfgValue));
//                cfgValue++;
//            }
//        } else if (MesPcnEnumUtil.PRINT_TEMPLATE.NO_SORT_BAR_CODE_GZ.getCode().equals(mesProduceSnPrintModel.getPrintTemplate()) || MesPcnEnumUtil.PRINT_TEMPLATE.NO_SORT_BAR_CODE_GZ_NEW.getCode().equals(mesProduceSnPrintModel.getPrintTemplate())) {
//            if (StringUtils.isEmpty(mesConfig.getCfgValueDesc()) || !TimeTool.getToday().equals(mesConfig.getCfgValueDesc())) {
//                cfgValue = 1;
//                mesConfig.setCfgValueDesc(TimeTool.getToday());
//            }
//            for (int i = 0; i < mesProduceSnPrintModel.getPrintQty(); i++) {
//                serialNoList.add(String.format("%05d", cfgValue));
//                cfgValue++;
//            }
//        }
//        mesConfig.setCfgValue(String.valueOf(cfgValue));
//        ConvertBean.serviceModelUpdate(mesConfig, mesProduceSnPrintModel.getUserName());
//        mesConfigRDao.update(mesConfig);
//        return serialNoList;
//    }

    private MesCustomerPart getMesCustomerPart(MesProduceSnPrintModel mesProduceSnPrintModel) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(mesProduceSnPrintModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesProduceSnPrintModel.getPartNo(), "erpPartNo", ddlPackBean);
        MesCustomerPart mesCustomerPart = mesCustomerPartRDao.getByProperty(ddlPackBean);
        if (Objects.isNull(mesCustomerPart)) {
            MesPcnException.throwMesBusiException("物料【%s】客户零件关系未维护", mesProduceSnPrintModel.getPartNo());
        }
        return mesCustomerPart;
    }

    private MesProduceSn generateMesProduceSn(MesPart mesPart, String sn, String userName, Double qty, String format) {
        MesProduceSn mesProduceSn = new MesProduceSn();
        mesProduceSn.setSerialNumber(snowflakeIdMaker.nextId() + "");
        mesProduceSn.setProductSn(sn);
        mesProduceSn.setCustSn(sn);
        mesProduceSn.setPartNo(mesPart.getPartNo());
        mesProduceSn.setPartName(mesPart.getPartName());
        mesProduceSn.setProcessLabelTemplate(mesPart.getProcessLabelTemplate());
        mesProduceSn.setCustLabelTemplate(mesPart.getCustLabelTemplate());
        mesProduceSn.setProdLabelTemplate(mesPart.getProductLabelTemplate());
        mesProduceSn.setQty(qty);
        mesProduceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.CREATE.getValue());
        mesProduceSn.setQcStatus(MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue());
        mesProduceSn.setLotNo(TimeTool.getNowTime(format));
        mesProduceSn.setPrintCount(MesPcnExtConstWords.ONE);
        mesProduceSn.setPrintStatus(MesExtEnumUtil.PRINT_STATUS.PRINTED.getValue());
        mesProduceSn.setOrganizeCode(mesPart.getOrganizeCode());
        ConvertBean.serviceModelInitialize(mesProduceSn, userName);
        return mesProduceSn;
    }

    private String getDateTime(Date date) {
        StringBuilder dateTime = new StringBuilder();
        String yearStr = MesExtEnumUtil.YEAR_SHORT.valueOfDescription(Integer.parseInt(TimeTool.getYear(date)));
        if (StringUtils.isEmpty(yearStr)) {
            MesPcnException.throwMesBusiException("【%s】年月简号不存在", TimeTool.getYear(date));
        }
        String monthStr = MesExtEnumUtil.MONTH_SHORT.valueOfDescription(Integer.parseInt(TimeTool.getMonth(date)));
        if (StringUtils.isEmpty(monthStr)) {
            MesPcnException.throwMesBusiException("【%s】月简号不存在", TimeTool.getMonth(date));
        }
        return dateTime.append(yearStr).append(monthStr).append(TimeTool.getDay(date)).toString();
    }


    private String stringFormat(String data, Integer length) {
        return stringFormat(data, length, true, "*");
    }

    private String stringFormat(String data, Integer length, boolean left, String symbol) {
        //数据长途大于填充位数直接返回
        if (data.length() >= length) {
            return data;
        }
        StringBuilder dataBuffer = new StringBuilder();
        //左拼接
        if (left) {
            dataBuffer.append(data);
        }
        for (int i = 0; i < length - data.length(); i++) {
            dataBuffer.append(symbol);
        }
        //右拼接
        if (!left) {
            dataBuffer.append(data);
        }
        return dataBuffer.toString();
    }

    // 34进制字符集
    private String toBase34(int number) {
        String base34Chars = "0123456789ABCDEFGHJKLMNPQRSTUVWXYZ";
        StringBuilder sb = new StringBuilder();

        while (number > 0) {
            int remainder = number % 34;
            sb.append(base34Chars.charAt(remainder));
            number /= 34;
        }
        return sb.reverse().toString();
    }
}
