package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesJisShippingService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesShippingLoadingCheckService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesShippingLoadingCheckModel;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesLoadingList;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesLoadingListDetail;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesShippingOrderManagement;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * @Description :
 * @Reference :
 * @Author :
 * @CreateDate : 2024-04-26 15:38
 * @Modify:
 **/
@Service
@Slf4j
public class MesShippingLoadingCheckService implements IMesShippingLoadingCheckService {

    @Autowired
    private MesLoadingListRepository vehiclesOrderRepository;
    @Autowired
    private MesLoadingListDetailRepository vehiclesOrderDetailRepository;

    @Autowired
    private MesShippingOrderManagementRepository shippingOrderManagementRepository;
    @Autowired
    private MesShippingOrderManagementDetailRepository shippingOrderManagementDetailRepository;
    @Autowired
    private MesProduceSnRepository produceSnRepository;
    @Autowired
    private IMesJisShippingService jisShippingService;
    @Autowired
    private MesJisShippingRepository jisShippingRepository;

    @Override
    public MesShippingLoadingCheckModel queryLoadingOrderNo(MesLoadingList loadingList) {


        //校验装车单
        MesLoadingList loadingListResult = checkLoadingList(loadingList.getOrderCode(), loadingList.getOrganizeCode());

        //若生成装车单，则一定有装车单明细
        DdlPackBean orderDetailPackBean = DdlPackBean.getDdlPackBean(loadingList.organizeCode);
        DdlPreparedPack.getNumEqualPack(loadingListResult.getId(), "pid", orderDetailPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue()}, new String[]{"seq"}, orderDetailPackBean);
        List<MesLoadingListDetail> detailList = vehiclesOrderDetailRepository.findByHqlWhere(orderDetailPackBean);

        //是否逆转
        if (loadingListResult.getIsReverse() == MesCommonConstant.TRUE_INTEGER) {
            // 逆转
            for (int i = 0; i < detailList.size(); i += loadingListResult.getReverseQuantity()) {
                if (i + loadingListResult.getReverseQuantity() - 1 < detailList.size()) {
                    Collections.reverse(detailList.subList(i, i + loadingListResult.getReverseQuantity()));
                }
            }
        }

        MesShippingLoadingCheckModel model = new MesShippingLoadingCheckModel();
        Long count = detailList.stream().filter(k -> k.getStatus() == MesExtEnumUtil.LOADING_ORDER_DETAIL_SHIPPING_STATUS.SHIPPINGED.getValue()).count();
        model.setIsScanCount(count.intValue());
        model.setOrderCode(loadingList.getOrderCode());

        //查询最近扫描完成的装车单号
        DdlPackBean recentlyOrderPackBean = DdlPackBean.getDdlPackBean(loadingList.organizeCode);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.MES_LOADING_STATUS.SCANNED.getValue(), "status", recentlyOrderPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"modifyDatetime"}, recentlyOrderPackBean);
        MesLoadingList recentlyVehiclesOrder = vehiclesOrderRepository.getByProperty(recentlyOrderPackBean);

        if (!StringUtils.isEmpty(recentlyVehiclesOrder)) {

            model.setRecentlyScanVehiclesOrder(recentlyVehiclesOrder.getOrderCode());
        }

        detailList.forEach(k -> {
            if (k.getStatus() == MesExtEnumUtil.LOADING_ORDER_DETAIL_SHIPPING_STATUS.SHIPPINGED.getValue()){
                k.setScanSn(k.getShippingCode());
                k.setTempStatus(MesCommonConstant.TRUE_INTEGER);
                k.setTempStatusVal("扫描完成");
            }
        });

        model.setDetailList(detailList);

        return model;
    }

    @Override
    public MesLoadingList queryLoadingOrderNoByAutoPushOrder(String shippingGroupNo, String org) {


        List<Integer> statusList = new ArrayList<>();
        statusList.add(MesExtEnumUtil.MES_LOADING_STATUS.CREATE.getValue());
        statusList.add(MesExtEnumUtil.MES_LOADING_STATUS.PUBLISH.getValue());
        statusList.add(MesExtEnumUtil.MES_LOADING_STATUS.CREATE_ASN.getValue());
        statusList.add(MesExtEnumUtil.MES_LOADING_STATUS.LOADING.getValue());

        //查询最近需要扫描的装车单号
        DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getInPackList(statusList, "status", orderPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue()}, new String[]{"seq"}, orderPackBean);
        MesLoadingList loadingList = vehiclesOrderRepository.getByProperty(orderPackBean);

        return loadingList;
    }

    @Override
    public MesShippingLoadingCheckModel saveVehiclesOrderAndDetail(MesShippingLoadingCheckModel model, String org) {

        //校验装车单
        MesLoadingList loadingList = checkLoadingList(model.getOrderCode(), org);

        //校验发运单是否存在
        DdlPackBean orderDetailPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getNumEqualPack(loadingList.getId(), "pid", orderDetailPackBean);
        DdlPreparedPack.getStringEqualPack(model.getShippingCode(), "shippingCode", orderDetailPackBean);
        MesLoadingListDetail vehiclesOrderDetail = vehiclesOrderDetailRepository.getByProperty(orderDetailPackBean);
        if (StringUtils.isEmpty(vehiclesOrderDetail)) {
            throw new ImppBusiException(String.format("【%s】此发运单不存在或还未生成装车单明细，请检查数据！", model.getShippingCode()));
        }

        //根据扫描的发运单以及前端传的发运单明细，进行匹配，若没有找到扫描的发运单，则提示扫描的发运单号不在此装车单中，请检查数据！。
        Boolean scanflg = false;
        for (MesLoadingListDetail loadingListDetail : model.getDetailList()) {
            if (loadingListDetail.getShippingCode().equals(model.getShippingCode())) {

                if (loadingListDetail.getTempStatus() == MesCommonConstant.TRUE_INTEGER) {
                    throw new ImppBusiException(String.format("【%s】此发运单已经扫描过，请检查数据！", model.getShippingCode()));
                }

                //判断是否需要排序校验 默认排序则需要校验
                if (loadingList.getScanSeqWay() == MesExtEnumUtil.SHIPPING_GROUP_SCAN_CONFIRM_SEQ_MODE.DEFAULT_SORT.getValue()) {
                    //获取明细中最小的

                    Long minSeq = model.getDetailList().stream().filter(k -> StringUtils.isEmpty(k.getScanSn())).mapToLong(k -> k.getSeq()).min().getAsLong();
                    //若扫描的不是最小顺序 则报错
                    if (minSeq.longValue() != loadingListDetail.getSeq().longValue()) {
                        throw new ImppBusiException(String.format("【%s】此发运单扫描顺序错误，请检查数据！", model.getShippingCode()));
                    }
                }

                loadingListDetail.setScanSn(model.getShippingCode());
                loadingListDetail.setTempStatus(MesCommonConstant.TRUE_INTEGER);
                loadingListDetail.setTempStatusVal("扫描完成");

                //根据是否启动记忆判断 若启用记忆，则根据保存数据库
                if (loadingList.getIsEnableMemory() == MesCommonConstant.TRUE_INTEGER) {

                    //若装车单状态为创建或发布已生成ASN 则写入开始扫描时间
                    if (loadingList.getStatus() == MesExtEnumUtil.MES_LOADING_STATUS.CREATE.getValue() ||
                            loadingList.getStatus() == MesExtEnumUtil.MES_LOADING_STATUS.PUBLISH.getValue() ||
                            loadingList.getStatus() == MesExtEnumUtil.MES_LOADING_STATUS.CREATE_ASN.getValue()) {
                        loadingList.setStartScanTime(TimeTool.getNowTime(true));
                    }

                    loadingList.setStatus(MesExtEnumUtil.MES_LOADING_STATUS.LOADING.getValue());
                    ConvertBean.serviceModelUpdate(loadingList, AuthUtil.getSessionUser().getUserName());
                    vehiclesOrderRepository.save(loadingList);

                    loadingListDetail.setStatus(MesExtEnumUtil.LOADING_ORDER_DETAIL_SHIPPING_STATUS.SHIPPINGED.getValue());
                    ConvertBean.serviceModelUpdate(loadingListDetail, AuthUtil.getSessionUser().getUserName());
                    vehiclesOrderDetailRepository.save(loadingListDetail);
                }

                //根据扫描的装车单找到是否先装车配置
                //若配置为否，则查询装车单中所有的发运单是否都进行【排序发运校验】完成，若【排序发运校验】没完成，则提示，先进行排序发运校验。
                if (loadingList.getIsFirstInstall() == MesCommonConstant.FALSE_INTEGER) {
                    MesShippingOrderManagement orderManagement = shippingOrderManagementRepository.getById(loadingListDetail.getShippingId());
                    if (!StringUtils.isEmpty(orderManagement)) {
                        throw new ImppBusiException(String.format("【%s】此装车单中发运单为【%s】，还未做完【排序发运校验】，请检查数据", model.getShippingCode(), loadingListDetail.getShippingCode()));
                    }
                }
                scanflg = true;
                break;
            }
        }

        if (!scanflg) {
            throw new ImppBusiException(String.format("【%s】扫描的发运单号不在【%s】此装车单中，请检查数据！", model.getShippingCode(), model.getOrderCode()));
        }

        Long count = model.getDetailList().stream().filter(k -> k.getTempStatus() == MesCommonConstant.TRUE_INTEGER).count();
        model.setIsScanCount(count.intValue());

        //全部扫描完成
        if (count.intValue() == model.getDetailList().size()) {

            //若装车单状态为创建或发布已生成ASN 则写入开始扫描时间
            if (loadingList.getStatus() == MesExtEnumUtil.MES_LOADING_STATUS.CREATE.getValue() ||
                    loadingList.getStatus() == MesExtEnumUtil.MES_LOADING_STATUS.PUBLISH.getValue() ||
                    loadingList.getStatus() == MesExtEnumUtil.MES_LOADING_STATUS.CREATE_ASN.getValue()) {
                loadingList.setStartScanTime(TimeTool.getNowTime(true));
            }
            loadingList.setEndScanTime(TimeTool.getNowTime(true));
            //装车单主表
            loadingList.setStatus(MesExtEnumUtil.MES_LOADING_STATUS.SCANNED.getValue());
            ConvertBean.serviceModelUpdate(loadingList, AuthUtil.getSessionUser().getUserName());

            //装车单明细
            DdlPackBean vehiclesOrderDetailPackBean = DdlPackBean.getDdlPackBean(org);
            DdlPreparedPack.getNumEqualPack(loadingList.getId(), "pid", vehiclesOrderDetailPackBean);
            List<MesLoadingListDetail> vehiclesOrderDetailList = vehiclesOrderDetailRepository.findByHqlWhere(vehiclesOrderDetailPackBean);
            vehiclesOrderDetailList.forEach(k -> {
                k.setStatus(MesExtEnumUtil.LOADING_ORDER_DETAIL_SHIPPING_STATUS.SHIPPINGED.getValue());
                ConvertBean.serviceModelUpdate(k, AuthUtil.getSessionUser().getUserName());
            });

            vehiclesOrderDetailRepository.saveAll(vehiclesOrderDetailList);
            vehiclesOrderRepository.save(loadingList);
        }
        return model;
    }

    /**
     * 校验装车单
     *
     * @param orderCode
     * @param org
     */
    public MesLoadingList checkLoadingList(String orderCode, String org) {

        //校验装车单是否存在
        DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(orderCode, "orderCode", orderPackBean);
        MesLoadingList loadingList = vehiclesOrderRepository.getByProperty(orderPackBean);
        if (StringUtils.isEmpty(loadingList)) {
            throw new ImppBusiException(String.format("【%s】此装车单不存在，请检查数据！", orderCode));
        }

        //校验装车单状态 必须是未装车条码
        if (loadingList.getStatus() != MesExtEnumUtil.MES_LOADING_STATUS.CREATE.getValue() &&
                loadingList.getStatus() != MesExtEnumUtil.MES_LOADING_STATUS.PUBLISH.getValue() &&
                loadingList.getStatus() != MesExtEnumUtil.MES_LOADING_STATUS.CREATE_ASN.getValue() &&
                loadingList.getStatus() != MesExtEnumUtil.MES_LOADING_STATUS.LOADING.getValue()) {
            throw new ImppBusiException(String.format("【%s】此装车单状态为【%s】,请选择状态为【%s】或状态【%s】或状态【%s】或状态【%s】的装车单", loadingList.getOrderCode(),
                    MesExtEnumUtil.MES_LOADING_STATUS.valueOfDescription(loadingList.getStatus()),
                    MesExtEnumUtil.MES_LOADING_STATUS.CREATE.getDescription(),
                    MesExtEnumUtil.MES_LOADING_STATUS.PUBLISH.getDescription(),
                    MesExtEnumUtil.MES_LOADING_STATUS.CREATE_ASN.getDescription(),
                    MesExtEnumUtil.MES_LOADING_STATUS.LOADING.getDescription()));
        }
        //增加发运顺序卡控，查询是否存在小于当前装车单对应的seq，且发运状态在（“创建”，“已确认”，“已生成ASN”，“装车中”）的装车单数据
        List<Integer> loadingListStatusList = new ArrayList<>();
        loadingListStatusList.add(MesExtEnumUtil.MES_LOADING_STATUS.CREATE.getValue());
        loadingListStatusList.add(MesExtEnumUtil.MES_LOADING_STATUS.PUBLISH.getValue());
        loadingListStatusList.add(MesExtEnumUtil.MES_LOADING_STATUS.CREATE_ASN.getValue());
        loadingListStatusList.add(MesExtEnumUtil.MES_LOADING_STATUS.LOADING.getValue());
        orderPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringSmallerNotEqualPack(loadingList.getSeq(), "seq", orderPackBean);
        DdlPreparedPack.getInPackList(loadingListStatusList, "status", orderPackBean);
        int count = vehiclesOrderRepository.findByHqlWhereCount(orderPackBean);
        if (count != 0) {
            throw new ImppBusiException(String.format("【%s】装车单对应排序【%s】前存在未扫描发运的装车单，请检查数据！",
                    orderCode, loadingList.getSeq()));
        }
        return loadingList;
    }

}
