package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.rulematch;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesNumberRuleMatchDispatchService;
import com.google.common.base.Objects;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 正则表达式匹配
 */
@Slf4j
@Service
public class MesNumberRuleMatchRegularExpressionService implements IMesNumberRuleMatchDispatchService {

    @Override
    public Boolean matchNumberRule(String organizeCode, String sn, Object... params) {

        String matchRule =  params[1].toString();

        return match(matchRule, sn);

    }

    private boolean match(String ruleStr, String barCode) {
        if (StringUtils.isEmpty(ruleStr)) {
            return false;
        }
        if (!ruleStr.contains("*") && !ruleStr.contains("?") && !ruleStr.contains(",")) {
            if (Objects.equal(ruleStr.toLowerCase(), barCode.toLowerCase())) {
                return true;
            }
            return false;
        }
        String ruleArray[] = ruleStr.split(",");
        for (String rule : ruleArray) {
            if (rule.contains("yyyyMMdd")) {
                String pattern = "\\d{4}\\d{2}\\d{2}";
                Pattern regex = Pattern.compile(pattern);
                Matcher matcher = regex.matcher(rule);
                return  matcher.matches();
            } else if (rule.contains("yyyy.MM.dd") || rule.contains("yyyy/MM/dd") || rule.contains("yyyy-MM-dd")) {
                String pattern = "\\d{4}[./-]\\d{2}[./-]\\d{2}";
                Pattern regex = Pattern.compile(pattern);
                Matcher matcher = regex.matcher(rule);
                return  matcher.matches();
            }

            String ruleNoSpecialChar = transfer(rule);
            String barcodeNoSpecialChar = transfer(barCode);
            ///将规则转换为正则表达式，使用正则表达式进行匹配，正则中去除二维码的起始特殊符号
            //update by huxj 2020.04.07 添加二维码前缀正则匹配
            String regStr = "^" + ruleNoSpecialChar.toLowerCase().replace(")", "\\)").replace("?", "(?:.|\n){1}").replace("*", "(?:.|-\n)*").replace("[", "\\[") + "$";
            //Regex rgx = new Regex("^" + ruleNoSpecialChar.ToLower().Replace("?", @"(?:.|\n){1}").Replace("*", @"(?:.|\n)*") + "$");
            Pattern regex = Pattern.compile(regStr);
            //end update
            if (regex.matcher(barcodeNoSpecialChar.toLowerCase()).matches()) return true;
            if (rule == barCode) return true;

        }
        return false;
    }

    public String transfer(String str){
        // 1. 定义正则表达式，匹配特殊字符
        String regex = "[^a-zA-Z0-9^\\s^*?[)>.:^/]]";

        // 2. 使用正则表达式替换特殊字符为空字符串
       return str.replaceAll(regex, "");

    }

    public static void main(String[] args) {
        String sn = "1574110000123";
        String matchRule = "1574110000*";
        System.out.println(sn.matches(matchRule));
    }
}
