package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import akka.actor.ActorRef;
import akka.actor.ActorSystem;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesNcProcessingService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesCellEquipContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionProcessContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionPsOutContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.ActorMessage;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesNcProcessingInputModel;
import cn.estsh.i3plus.mes.pcn.config.SpringExtProvider;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.codemaker.SnowflakeIdMaker;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MesEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesDefectType;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspection;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspectionDetail;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Objects;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @Description : 加工异常处理工步  注释调整
 * @Author : zxw
 **/
@Slf4j
@Service
public class MesProductResultErrorHandleStepService extends BaseStepService {
    @Autowired
    private SnowflakeIdMaker snowflakeIdMaker;

    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Autowired
    private IMesNcProcessingService ncProcessingService;

    @Autowired
    private IMesProductionDispatchContextStepService mesProductionDispatchContextStepService;

    public static final String ORDER_NO_JIS_SORT = "ORDER_NO_JIS_SORT";

    private static final Map<String, ActorRef> refMap = new ConcurrentHashMap<>(200);

    @Override
    public StepResult execute(StationRequestBean reqBean) {

        StepResult stepResult = StepResult.getSuccessComplete();
        StationResultBean resultBean = new StationResultBean();

        // 获取加工结果
        String result = mesProductionDispatchContextStepService.getProductResultContext(reqBean);
        // 如果是报废或者可疑，需要编辑条码状态为不合格、并且nc判定需要新增一条记录
        if (StringUtils.isEmpty(result) || Objects.equal(MesExtEnumUtil.EQUIP_VARIABLE_CFG_CATEGORY.PRODUCT_OK.getCode(), result)) {
            return execSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, "加工结果合格,无需记录异常");
        }
        // 获取条码列表
        List<MesProductionPsOutContext> mesProduceSns = mesProductionDispatchContextStepService.getProductionPsOutContext(reqBean);
        // 保存加工异常记录(NC判断记录)
        saveProductResultException(reqBean, mesProduceSns, result);

        return execSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(), stepResult, "加工异常处理成功");


   }

    /**
     * 存加工异常记录
     * @param reqBean
     * @param mesProduceSns
     */
    private void saveProductResultException(StationRequestBean reqBean, List<MesProductionPsOutContext> mesProduceSns, String result) {
        String defectTypeCode = MesEnumUtil.DEFECT_TYPE_CODE.DETERMIND.getValue();
        mesProduceSns.forEach(mesProduceSn -> {
            MesNcProcessingInputModel model = new MesNcProcessingInputModel();
            MesDefectType mesDefectType = new MesDefectType();
            mesDefectType.setBackDefect(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
            mesDefectType.setDefectTypeCode(defectTypeCode);
            mesDefectType.setDefectTypeName(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getDescription());
            //mesDefectType.setDefectType(MesExtEnumUtil.MES_DEFECT_TYPE);
            model.setSn(mesProduceSn.getProductSn());
            model.setPerson(mesDefectType);

            MesPartInspection partInspection = new MesPartInspection();

            Long id = snowflakeIdMaker.nextId();
            partInspection.setWorkCenterCode(reqBean.getWorkCenterCode());
            partInspection.setSourceType(MesExtEnumUtil.PART_INSPECTION_SOURCE_TYPE.SINGLE.getValue());
            partInspection.setPartNo(mesProduceSn.getPartNo());
            partInspection.setPartName(mesProduceSn.getPartName());
            partInspection.setDefectTypeCode(mesDefectType.getDefectTypeCode());
            partInspection.setInspectionDate(DateUtil.format(new Date(), DatePattern.NORM_DATETIME_MS_PATTERN));
            partInspection.setLotNo(mesProduceSn.getLotNo());
            if (!Objects.equal(MesExtEnumUtil.EQUIP_VARIABLE_CFG_CATEGORY.PRODUCT_SCRAP.getValue(), result)) {
                partInspection.setNcStatus(MesExtEnumUtil.PART_INSPECTION_NC_STATUS.SCRAP.getValue());
            } else {
                partInspection.setNcStatus(MesExtEnumUtil.PART_INSPECTION_NC_STATUS.CREATE.getValue());
            }
            partInspection.setRejectQty(mesProduceSn.getQty().intValue());
            partInspection.setShiftCode(mesProduceSn.getShiftCode());
            partInspection.setSn(mesProduceSn.getProductSn());
            partInspection.setQty(mesProduceSn.getQty().intValue());
            partInspection.setInspectionStatus(MesExtEnumUtil.PART_INSPECTION_STATUS.FAIL.getValue());
            partInspection.setId(id);
            ConvertBean.serviceModelInitialize(partInspection, reqBean.getUserInfo());

            MesPartInspectionDetail mesPartInspectionDetail = new MesPartInspectionDetail();
            mesPartInspectionDetail.setPid(id);
            mesPartInspectionDetail.setStatus(MesExtEnumUtil.PART_INSPECTION_NC_STATUS.CREATE.getValue());
            mesPartInspectionDetail.setDefectTypeCode(mesDefectType.getDefectTypeCode());
            mesPartInspectionDetail.setDefectTypeName(mesDefectType.getDefectTypeName());
            model.setPartInspection(partInspection);
            model.setPartInspectionDetailList(Arrays.asList(mesPartInspectionDetail));
            ncProcessingService.saveNc(model, reqBean.getOrganizeCode());
        });
    }
}
